import {
    Http, Loader, Metadata,
} from '@zeedhi/core';
import {
    Modal, ModalService, Text, Form, LoadingService, Number, Select, IGridEvent, TextInput, DialogService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class TabStandardFinancialIndicesController {
    private addStandardFinancialIndices!: Modal;

    public BPController = Loader.getInstance('BusinessPartnerController');

    public async openModalStandardFinancialIndices() {
        this.addStandardFinancialIndices = await ModalService.createFromJson('sources/BusinessPartner/Modal/modal-add-standardFinancialIndices', true);
        this.addStandardFinancialIndices.show();
    }

    public async openModalInsert() {
        await this.openModalStandardFinancialIndices();
        this.getSomaPercentualIndicesParceiro().then((response) => {
            const { percentual } = response.data.dataset.retorno;
            Metadata.getInstance<TextInput>('NRPERCENTINDICE').value = 100 - percentual;
        });
    }

    public async openModalEdit({ row }: IGridEvent) {
        await this.openModalStandardFinancialIndices();
        Metadata.getInstance<TextInput>('NRINDICESPADRPARC').value = row.NRINDICESPADRPARC;
        Metadata.getInstance<Select>('CDINDICE').setValue(row.CDINDICE);
        Metadata.getInstance<Number>('NRPERCENTINDICE').value = row.NRPERCENTINDICE;
    }

    /**
     * Retorna a soma do percentual dos indices financeiros cadastrados para o parceiro de negocio atual,
     * ignorando o percentual do registro informado no parametro.
     *
     * @param nrindicespadrparc
     * @returns float
     */
    public getSomaPercentualIndicesParceiro(nrindicespadrparc = null) {
        return Http.post('/getSomaPercentualIndicesParceiro', {
            NRPARCNEGOCIO: this.BPController.currentRow.NRPARCNEGOCIO,
            NRINDICESPADRPARC: nrindicespadrparc,
        });
    }

    public closeModalStandardFinancialIndices() {
        this.addStandardFinancialIndices.hide();
        this.addStandardFinancialIndices?.destroy();
    }

    public onMountNameModal() {
        Metadata.getInstance<Text>('nameOfCorporate-addStandarFinancialIndices').text = this.BPController.NAMEOFCORPORATE;
    }

    public salvarIndice() {
        const formIndices = Metadata.getInstance<Form>('formAddFinancialIndices');
        const grid = Metadata.getInstance<TekGrid>('tekgrid_tabStandardFinancialIndices');

        if (formIndices.validate()) {
            const nrparcnegocio = this.BPController.currentRow.NRPARCNEGOCIO;
            const indice = Metadata.getInstance<Select>('CDINDICE').selectValue;
            const nrindicespadrparc = Metadata.getInstance<TextInput>('NRINDICESPADRPARC').value;
            const nrpercentindice = Metadata.getInstance<Number>('NRPERCENTINDICE').value;

            LoadingService.show();

            this.validarIndiceDuplicadoParceiro(indice, nrparcnegocio, nrindicespadrparc)
                .then(() => this.getSomaPercentualIndicesParceiro(nrindicespadrparc))
                .then((response) => {
                    const { percentual } = response.data.dataset.retorno;
                    if (nrpercentindice + percentual <= 100) {
                        return Http.post('/salvarIndiceFinanceiroPadraoParceiro', {
                            NRINDICESPADRPARC: nrindicespadrparc,
                            NRPARCNEGOCIO: nrparcnegocio,
                            CDTPINDICE: indice.CDTPINDICE,
                            IDPERVALOR: indice.IDPERVALOR,
                            CDINDICE: indice.CDINDICE,
                            NRPERCENTINDICE: nrpercentindice,
                        });
                    }
                    return new Promise((resolve, reject) => {
                        reject(new Error('A soma dos percentuais de utilização dos Índices Financeiros não deve ultrapassar 100%.'));
                    });
                })
                .then(() => {
                    Utilities.showSuccessMessage();
                    this.closeModalStandardFinancialIndices();
                    grid.datasource.get();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    LoadingService.hide();
                });
        } else {
            Utilities.showRequiredFieldsMessage();
        }
    }

    public validarIndiceDuplicadoParceiro(indice: any, nrparcnegocio: number, nrindicespadrparc: number) {
        return Http.post('/validarIndiceDuplicadoParceiro', {
            CDINDICE: indice.CDINDICE,
            CDTPINDICE: indice.CDTPINDICE,
            IDPERVALOR: indice.IDPERVALOR,
            NRPARCNEGOCIO: nrparcnegocio,
            NRINDICESPADRPARC: nrindicespadrparc,
        });
    }

    public deleteIndice({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'TabStandardFinancialIndicesController.excluirIndices');
    }

    public async excluirIndices() {
        const grid = Metadata.getInstance<TekGrid>('tekgrid_tabStandardFinancialIndices');
        const ids = (grid.selectedRows.map((element: any) => element.NRINDICESPADRPARC));

        LoadingService.show();
        await Http.post('/excluirIndiceFinanceiroPadraoParceiro', {
            NRINDICESPADRPARC: ids,
            NRPARCNEGOCIO: this.BPController.currentRow.NRPARCNEGOCIO,
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
            grid.selectedRows = [];
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
